/*  mod_proxy_filter_xff extension module for Apache 2.2

    Copyright (C) 2011 Peter Pramberger

    Permission is hereby granted, free of charge, to any person obtaining
    a copy of this software and associated documentation files (the
    "Software"), to deal in the Software without restriction, including
    without limitation the rights to use, copy, modify, merge, publish,
    distribute, sublicense, and/or sell copies of the Software, and to
    permit persons to whom the Software is furnished to do so, subject
    to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*  Apache 2.2 extension module to filter X-Forwarded-* headers added by
    mod_proxy in reverse mode.
 */

#include "httpd.h"
#include "http_config.h"
#include "http_log.h"
#include "mod_proxy.h"

#define MOD_NAME "ProxyFilterXFF"
#define MOD_VER  "0.3"

module AP_MODULE_DECLARE_DATA proxy_filter_xff_module;


/* Module config */

typedef struct
{
  int bFilterEnable;
  int bFilterFull;
} PFXFFConfig;

static void *
PFXFF_config_init (apr_pool_t * p, server_rec * s)
{
  PFXFFConfig *pConfig = apr_pcalloc (p, sizeof (*pConfig));

  pConfig->bFilterEnable = 0;
  pConfig->bFilterFull = 0;

  return pConfig;
}

static const char *
PFXFF_config_FilterEnable (cmd_parms * cmd, void *dummy, int arg)
{
  PFXFFConfig *pConfig = ap_get_module_config (cmd->server->module_config,
					       &proxy_filter_xff_module);

  pConfig->bFilterEnable = arg;

  return NULL;
}

static const char *
PFXFF_config_FilterFull (cmd_parms * cmd, void *dummy, int arg)
{
  PFXFFConfig *pConfig = ap_get_module_config (cmd->server->module_config,
					       &proxy_filter_xff_module);

  pConfig->bFilterFull = arg;

  return NULL;
}

static const command_rec PFXFF_cmds[] = {
  AP_INIT_FLAG ("ProxyFilterXFEnable", PFXFF_config_FilterEnable, NULL,
		RSRC_CONF,
		"Enable filtering of X-Forwarded headers for this server "
		"(`on', `off')"),
  AP_INIT_FLAG ("ProxyFilterXFFull", PFXFF_config_FilterFull, NULL, RSRC_CONF,
		"Filter any X-Forwarded header, not only X-Forwarded-For "
		"(`on', `off')"),
  {NULL}
};


/* Filter handler */

static int
PFXFF_post_config (apr_pool_t * pconf, apr_pool_t * plog, apr_pool_t * ptemp,
		   server_rec * s)
{
  int init = 0;
  void *flag = NULL;
  server_rec *pS = NULL;

  /* Check if we're here for the first time */
  apr_pool_userdata_get (&flag, "PFXFF-init", s->process->pool);
  if (flag == NULL)
    {
      init = 1;
      apr_pool_userdata_set ((const void *) 1, "PFXFF-init",
			     apr_pool_cleanup_null, s->process->pool);
    }

  if (init)
    {
      /* Only print our signature if enabled somewhere */
      for (pS = s; pS; pS = pS->next)
	{
	  PFXFFConfig *pConfig = ap_get_module_config (pS->module_config,
						       &proxy_filter_xff_module);

	  if (pConfig->bFilterEnable == 1)
	    {
	      ap_log_error (APLOG_MARK, APLOG_NOTICE, 0, s,
			    "%s/%s configured.", MOD_NAME, MOD_VER);
#ifndef DEBUG
	      break;
#else
	      ap_log_error (APLOG_MARK, APLOG_DEBUG, 0, s,
			    "%s enabled for %s.", MOD_NAME,
			    pS->server_hostname);
#endif
	    }
	}
    }

  return OK;
}

static int
PFXFF_filter_handler (request_rec * r)
{
  PFXFFConfig *pConfig = ap_get_module_config (r->server->module_config,
					       &proxy_filter_xff_module);

  /* We're only interested if enabled and in reverse mode */
  if (PROXYREQ_REVERSE != r->proxyreq)
    return DECLINED;

  if (pConfig->bFilterEnable != 1)
    return DECLINED;

#ifdef DEBUG
  ap_log_rerror (APLOG_MARK, APLOG_DEBUG, 0, r,
		 "proxy_filter_xff: removing X-Forwarded-For%s header (%s): %s",
		 (pConfig->bFilterFull ? "/Host/Server" : ""),
		 r->server->server_hostname, r->connection->remote_ip);
#endif

  apr_table_unset (r->headers_in, "X-Forwarded-For");

  if (pConfig->bFilterFull == 1)
    {
      apr_table_unset (r->headers_in, "X-Forwarded-Host");
      apr_table_unset (r->headers_in, "X-Forwarded-Server");
    }

  return OK;
}

static void
PFXFF_register_hooks (apr_pool_t * p)
{
  ap_hook_post_config (PFXFF_post_config, NULL, NULL, APR_HOOK_LAST);
  APR_OPTIONAL_HOOK (proxy, fixups, PFXFF_filter_handler, NULL, NULL,
		     APR_HOOK_LAST);
}


/* Module stuff */

module AP_MODULE_DECLARE_DATA proxy_filter_xff_module = {
  STANDARD20_MODULE_STUFF,
  NULL,				/* create per-directory config structure */
  NULL,				/* merge per-directory config structures */
  PFXFF_config_init,		/* create per-server config structure */
  NULL,				/* merge per-server config structures */
  PFXFF_cmds,			/* command table */
  PFXFF_register_hooks		/* register hooks */
};
